#ExtraLargeIconGrid{
    width: 150px;
    height: 150px;
    text-align: center;
    border-radius: 8px;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
}
#LargeIconGrid{
    width: 95px;
    height: 95px;
    text-align: center;
    border-radius: 8px;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
}
#MediumIconGrid{
    width: 87px;
    height: 87px;
    text-align: center;
    border-radius: 8px;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
}
#SmallIconGrid{
    width: 80px;
    height: 80px;
    text-align: center;
    border-radius: 8px;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
}
#LargeRectIconGrid{
    width: 95px;
    height: 85px;
    text-align: center;
    border-radius: 8px;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
}
#MediumRectIconGrid{
    width: 92px;
    height: 78px;
    text-align: center;
    border-radius: 8px;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
}
#SmallRectIconGrid{
    width: 85px;
    height: 70px;
    text-align: center;
    border-radius: 8px;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
}

#ArcSearchEntry{
    min-height: 0px;
    border-radius: 8px;
    border-width: 1px;
    padding: 7px 9px;
}
#ArcSearchEntryRound{
    min-height: 0px;
    border-radius: 18px;
    border-width: 1px;
    padding: 7px 12px;
}

.arcmenu-search-top{
    margin: 0px 10px 6px 10px;
}

.arcmenu-search-bottom{
    margin: 6px 10px 0px 10px;
}

.symbolic-icons{
    -st-icon-style: symbolic;
}
.regular-icons{
    -st-icon-style: regular;
}

.arcmenu-menu-item{
    padding-top: 6px;
    padding-bottom: 6px;
}

#arcmenu-launcher-button{
    max-width: 90px;
    padding: 10px 14px;
    margin: 0px;
}
#arcmenu-launcher-button.active-item, #arcmenu-launcher-button:active{
    background-color: rgba(245, 245, 245, .08);
}

#arcmenu-plasma-button-top{
    width: 90px;
    height: 65px;
    border-radius: 8px;
    text-align: center;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
    border-top-width: 2px;
    border-color: transparent;
}
#arcmenu-plasma-button-top:active-item, .arcmenu-plasma-button-top:active{
    background-color: rgba(245, 245, 245, .05);
    border-top-width: 2px;
    border-color: rgb(61, 174, 235);
}
#arcmenu-plasma-button-bottom{
    width: 90px;
    height: 65px;
    border-radius: 8px;
    text-align: center;
    padding: 5px;
    spacing: 0px;
    margin: 0px;
    border-bottom-width: 2px;
    border-color: transparent;
}
#arcmenu-plasma-button-bottom:active-item, .arcmenu-plasma-button-bottom:active{
    background-color: rgba(245, 245, 245, .05);
    border-bottom-width: 2px;
    border-color: rgb(61, 174, 235);
}

StScrollView .small-vfade{
    -st-vfade-offset: 44px;
}

.arcmenu-margin-box{
    margin: 0px 6px;
}

.arcmenu-menu .popup-menu-ornament{
    width: 0;
}

.arcmenu-button{
    -st-icon-style: symbolic;
    border-width: 1px;
    border-radius: 8px;
    padding: 8px;
}

.arcmenu-small-button{
    -st-icon-style: symbolic;
    border-radius: 8px;
    padding: 3px 8px;
    margin: 0px;
}
.arcmenu-small-button:hover{
    background-color: rgba(255, 255, 255, 0.1);
}

.arcmenu-avatar-round{
    background-size: contain;
    border-radius: 999px;
}
.arcmenu-avatar-square{
    background-size: contain;
    border-radius: 0px;
}

.arcmenu-tooltip{
    border-radius: 8px;
    border: 1px solid rgba(255, 255, 255, 0.1);
    box-shadow: 0 2px 4px 0 rgba(0, 0, 0, 0.1);
    padding: 3px 8px;
    max-width: 550px;
    min-height: 0px;
}

/* Recently Installed App Indicator */
.arcmenu-indicator{
    color: rgb(41, 165, 249);
}

/* Recently Installed App Indicator in parent category */
.arcmenu-text-indicator{
    background-color: rgba(196, 196, 196, 0.3);
}

/* Style varius elements of ArcMenu below */
/* Uncomment each desired section and fill in desired options */
/* -----------------------------------------------------------*/
/* -----------------------------------------------------------*/
/* -----------------------------------------------------------*/

/* Menu Styling Section -----------------------------------------------------------------------------*/
/* See https://gitlab.gnome.org/GNOME/gnome-shell/-/blob/main/data/theme/gnome-shell-sass/widgets/_popovers.scss for more info*/
/*
.arcmenu-menu {
    color: ;
    font-size: ;
}
*/

/*
.arcmenu-menu .popup-menu-content {
    padding-top: 6px;
    padding-bottom: 8px;
    border-radius: ; 
    border: ; 
    box-shadow: ;
    background-color: ; 
}
*/

/*
.arcmenu-menu .popup-menu-item {
    padding: 
    border-radius: ;
    transition: 0.2s all ease;
    background-color: transparent;
}

.arcmenu-menu .popup-menu-item:focus, .arcmenu-menu .popup-menu-item:hover  {
    background-color: ;
}

.arcmenu-menu .popup-menu-item:checked {
    background-color: ;
    margin-bottom: ;
    box-shadow: ;
    border-radius: ;
}

.arcmenu-menu .popup-menu-item:active {
    background-color: ;
    color: ;
}

.arcmenu-menu .popup-menu-item:insensitive {
    background-color: ;
    color: ;
}
*/
/*End Menu Styling Section----------------------------------------------------------------------*/


/* Separator Color */
/*
.separator-color-style{
    background-color: ;
}
*/

/* Button Foreground Color */
/*
.arcmenu-menu-button{
    color: ;
}
*/

/* Button Foreground Hover Color */
/*
.arcmenu-panel-menu:hover .arcmenu-menu-button{
    color: ;
}
*/

/* Button Foreground Active Color */
/*
.arcmenu-menu-button:active{
    color: ;
}
*/

/* Button Background Hover Color */
/* 
.arcmenu-panel-menu:hover{
    background-color: ;
}
*/

/* Button Background Active Color */
/*
.arcmenu-panel-menu:active{
    background-color: ;
}
*/

/* Button Border and Border Radius */
/*
.arcmenu-panel-menu{
    border-radius: ;
    border: ;
}
*/
